#ifndef FLYWEIGHT_FACTORY_H
#define FLYWEIGHT_FACTORY_H

#include <boost/shared_ptr.hpp>

#include <map>
#include <string>
#include <cassert>

template<class flyweight>
class flyweight_factory
{
public:

   typedef boost::shared_ptr<flyweight> flyweight_ptr;

   template<class concrete_flyweight>
   flyweight_ptr get_flyweight()
   {
      const std::string key(typeid(concrete_flyweight).name());
      
      typename pool_type::const_iterator existing_flyweight(pool.find(key));
      
      if(pool.end() != existing_flyweight) {
         return existing_flyweight->second;
      }
      else {
         flyweight_ptr new_flyweight(new concrete_flyweight);
         
         const bool inserted = pool.insert(std::make_pair(key, new_flyweight)).second;
         assert(inserted);
         
         return new_flyweight;
      }
   }

private:
   typedef std::map<std::string, flyweight_ptr> pool_type;
   pool_type pool;
};

#endif

