#ifndef STOP_WATCH_H
#define STOP_WATCH_H

#include "watch_access.h"
#include "started_state.h"
#include "stopped_state.h"

class stop_watch
{
 public:
  void start();

  void stop();

 private:

  // Meyers Item 39: Prefer public inheritance plus composition in favour of private inheritance.
  class state_controller : public watch_access
  {
      // States are shared among instances by making them static -> the decision is on 
      // the client side and not spread among the states themselves.
      started_state started;
      stopped_state stopped;
      
      watch_state* current_state;

    public:
      state_controller()
         : current_state(&stopped)
      {
      }

      // Hide the extra indirection for the client by using en masse delegation.
      watch_state* operator->() const
      {
         return current_state;
      }

      virtual void change_to_started()
      {
         current_state = &started;
      }
      
      virtual void change_to_stopped()
      {
         current_state = &stopped;
      }
    };
  
  state_controller state;
};

#endif


